/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin.ui.action;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.validator.annotations.RequiredStringValidator;
import com.opensymphony.xwork2.validator.annotations.Validation;
import org.wso2.wsas.admin.service.util.AdminAuthenticator;

import java.util.Map;

/**
 *
 */
@Validation
public class LoginAction extends ActionSupport {

    private static final long serialVersionUID = 8009090071152629790L;

    private String password;
    
    private String username;


    public String execute() throws Exception {
        if (this.doLogin()) {
            Map session = ActionContext.getContext().getSession();
            session.put("user", username);
            return SUCCESS;
        } else {
            addActionError(getText("invalid_user_password"));
            return ERROR;
        }
    }


    private boolean doLogin() {
        // Use WSAS database authenticator
        AdminAuthenticator authenticator = new AdminAuthenticator();
        return authenticator.authenticate(this.username, this.password);
    }


    @RequiredStringValidator(message = "", key = "username")
    public String getUsername() {
        return username;
    }

    @RequiredStringValidator(message = "", key = "password")
    public String getPassword() {
        return password;
    }


    public void setUsername(String username) {
        this.username = username;
    }


    public void setPassword(String password) {
        this.password = password;
    }

}
