/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin.ui.action;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.solutions.identity.IdentityProviderConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.admin.RealmConfigAdmin;
import org.wso2.solutions.identity.i18n.Messages;

public class SetDefaultUserStoreAction extends ManagedAction {

    private static final long serialVersionUID = 5197582050072207667L;

    private static Log log = LogFactory.getLog(SetDefaultUserStoreAction.class);

    private final static Messages messages = Messages
            .getInstance(IdentityProviderConstants.RESOURCES);

    private String configName;

    public String execute() throws Exception {

        if (configName != null && configName.trim().length() > 0) {
            try {
                RealmConfigAdmin admin = new RealmConfigAdmin();
                boolean userRegDisabled = admin.setDefaultRealmConfiguration(configName);
                
                if(userRegDisabled) {
                    this.addInfoMessage(getText("user_reg_disabled",
                            new String[] { configName }));
                }
                log.info(messages.getMessage("changedDefaultUserStore",
                        new String[] { configName }));
            } catch (IdentityProviderException e) {
                this.addErrorMessage(e.getMessage());
            }
        } else {
            return ERROR;
        }

        return SUCCESS;
    }

    public void setConfigName(String configName) {
        this.configName = configName;
    }

}
