/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin.ui.action;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.solutions.identity.IdentityProviderConstants;
import org.wso2.solutions.identity.admin.ParameterAdmin;

public class ShowUserManagementAction extends ManagedAction {

    private static final long serialVersionUID = -3406313561520890097L;

    private boolean allowUserRegistration;

    private boolean enableEmailVerification;

    private boolean enableOpenIDRegistration;

    private int port = 0;

    private String host = null;

    private String fromAddress = null;

    private String username = null;

    private String password = null;

    private static Log log = LogFactory.getLog(ShowUserManagementAction.class);

    @Override
    public String execute() throws Exception {
        try {
            ParameterAdmin admin = new ParameterAdmin();

            this.allowUserRegistration = admin
                    .getParameter(IdentityProviderConstants.PARAM_NAME_ALLOW_USER_REGISTRATION) != null;
            this.enableEmailVerification = admin
                    .getParameter(IdentityProviderConstants.PARAM_NAME_ENABLE_EMAIL_VERIFICATION) != null;
            this.enableOpenIDRegistration = admin
                    .getParameter(IdentityProviderConstants.PARAM_NAME_ENABLE_OPENID_REGISTRATION) != null;

            String value = admin
                    .getParameterValue(IdentityProviderConstants.PARAM_NAME_EMAIL_PORT);
            if (value != null) {
                this.port = Integer.parseInt(value);
            }
            this.host = admin
                    .getParameterValue(IdentityProviderConstants.PARAM_NAME_EMAIL_HOST);
            this.fromAddress = admin
                    .getParameterValue(IdentityProviderConstants.PARAM_NAME_EMAIL_FROM_ADDRESS);
            this.username = admin
                    .getParameterValue(IdentityProviderConstants.PARAM_NAME_EMAIL_USERNAME);
            this.password = admin
                    .getParameterValue(IdentityProviderConstants.PARAM_NAME_EMAIL_PASSWORD);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return ERROR;
        }

        return SUCCESS;
    }

    public boolean isAllowUserRegistration() {
        return allowUserRegistration;
    }

    public boolean isEnableEmailVerification() {
        return enableEmailVerification;
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getFromAddress() {
        return fromAddress;
    }

    public void setFromAddress(String fromAddress) {
        this.fromAddress = fromAddress;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void setAllowUserRegistration(boolean allowUserRegistration) {
        this.allowUserRegistration = allowUserRegistration;
    }

    public void setEnableEmailVerification(boolean enableEmailVerification) {
        this.enableEmailVerification = enableEmailVerification;
    }

    public boolean isEnableOpenIDRegistration() {
        return enableOpenIDRegistration;
    }

    public void setEnableOpenIDRegistration(boolean enableOpenIDRegistration) {
        this.enableOpenIDRegistration = enableOpenIDRegistration;
    }

}
