/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.admin.ui.action;

import org.wso2.solutions.identity.admin.ClaimsAdmin;
import org.wso2.solutions.identity.admin.ui.UIConstants;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;

public class UpdateClaimAction extends ManagedAction {

    private static final long serialVersionUID = 3006810147981917457L;

    private String claimUri;

    private String operation;

    public String execute() throws Exception {
        if (claimUri != null && operation != null && claimUri.length() > 0
                && operation.length() > 0) {
            ClaimsAdmin admin = new ClaimsAdmin();
            ClaimDO claimDO = admin.findClaimByURI(claimUri);
            claimDO.setUri(claimUri);
            if (operation.equals(UIConstants.CLAIM_UPDATE_DISABLE)) {
                claimDO.setSupported(false);
                claimDO.setAttrId(ClaimDO.DAFAULT_ATTR_ID);
                admin.updateClaim(claimDO);
            } else if (operation.equals(UIConstants.CLAIM_UPDATE_ENABLE)) {                
                if (claimDO.getOpenIDTag() != null) {
                    if (!admin.isValidOpenIDTag(claimDO.getOpenIDTag())) {
                        // You can't have two supported claims with the same OpenID tag
                        this.addErrorMessage(getText("add_claim_duplicate_openid_tag",
                                new String[] { claimDO.getOpenIDTag() }));
                        return ERROR;
                    }
                }       
                claimDO.setSupported(true);
                admin.updateClaim(claimDO);
            }
        }
        return SUCCESS;
    }

    public void setClaimUri(String claimUri) {
        this.claimUri = claimUri;
    }

    public void setOperation(String operation) {
        this.operation = operation;
    }

    public String getClaimUri() {
        return claimUri;
    }

    public String getOperation() {
        return operation;
    }
    
    

}
