/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.sts.saml;

import junit.framework.TestCase;

import org.apache.axiom.om.util.UUIDGenerator;
import org.apache.xml.security.Init;
import org.apache.xml.security.signature.XMLSignature;
import org.joda.time.DateTime;
import org.opensaml.DefaultBootstrap;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class SAML2TokenBuilderTest extends TestCase{

    static {
        Init.init();
    }

    public void testSAML2TokenCreation() throws Exception {

        DefaultBootstrap.bootstrap();

        X509CredentialImpl keyHolder = new X509CredentialImpl();

        SAML2TokenBuilderSimulator builder = new SAML2TokenBuilderSimulator();
        builder.createStatement(null, null);

        DateTime notBefore = new DateTime();
        DateTime notAfter = new DateTime(notBefore.getMillis() + (300 * 1000));
        String assertionId = UUIDGenerator.getUUID();

        builder.createSAMLAssertion(notAfter, notBefore, assertionId);
        builder.setSignature(XMLSignature.ALGO_ID_SIGNATURE_RSA, keyHolder);
        builder.marshellAndSign();
        Element elemSAML = builder.getSAMLasDOM();

        NodeList list = elemSAML.getElementsByTagNameNS("http://www.w3.org/2000/09/xmldsig#", "Signature");
        TestCase.assertNotNull(list);
        TestCase.assertEquals(1, list.getLength());
    }
}
