/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.relyingparty.openid;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.openid4java.infocard.InfocardException;
import org.openid4java.infocard.OpenIDToken;
import org.openid4java.message.ParameterList;
import org.wso2.solutions.identity.IdentityConstants;

public class OpenIDUtil {

    /**
     * Validates whether the given XMLToken corresponding to an OpenID
     * information card.
     * @param xmlToken OpenIDToken
     * @return true if validated as an OpenID information card, else false.
     */
    public static boolean isOpenIDInfoCardToken(String xmlToken) {
        try {
            if (xmlToken != null && xmlToken.trim().length() > 0)
                OpenIDToken.createFromXmlToken(xmlToken);
            else
                return false;
        } catch (InfocardException e) {
            return false;
        }

        return true;
    }

    /**
     * Validates whether the provided request is an OpenID authentication
     * request.
     * @param request HttpServletRequest
     * @return true if validated as an OpenID authentication, else false.
     */
    public static boolean isOpenIDAuthetication(HttpServletRequest request) {

        ParameterList response = null;
        Map map = null;

        try {
            map = request.getParameterMap();

            if (map == null)
                return false;

            response = new ParameterList(map);

            if (response.hasParameter(IdentityConstants.OpenId.ATTR_MODE)) {
                String mode = null;
                mode = response
                        .getParameterValue(IdentityConstants.OpenId.ATTR_MODE);

                if (IdentityConstants.OpenId.OPENOD_RESPONSE.equals(mode)
                        || IdentityConstants.OpenId.CANCEL.equals(mode)) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }

        } catch (Exception e) {
            return false;
        }
    }
}
