/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.StrutsStatics;
import org.wso2.solutions.identity.admin.ClaimsAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;
import org.wso2.solutions.identity.user.ui.UIConstants;

import com.opensymphony.xwork2.ActionContext;

public class AddUserProfileAction extends ManagedAction {

    /**
     * 
     */
    private static final long serialVersionUID = -3415602317665292286L;

    private List<ClaimDO> claims;

    /**
     * This will get executed while adding a new user profile to an existing
     * user and will load all the supported claims where the user can use in his
     * profile.
     */
    public String execute() throws Exception {

        String userName = null;
        HttpServletRequest request = null;
        ClaimsAdmin claimsAdmin = null;
        ClaimDO[] claimDOs = null;

        try {
            request = (HttpServletRequest) ActionContext.getContext().get(
                    StrutsStatics.HTTP_REQUEST);

            userName = (String) request.getSession().getAttribute(
                    UIConstants.USER);

            if (userName == null) {
                this.addErrorMessage(getText("invalid_user_login"));
                return ERROR;
            }

            claimsAdmin = new ClaimsAdmin();
            claimDOs = claimsAdmin.getAllMappedEnabledClaims();

            claims = new ArrayList<ClaimDO>();
            for (int i = 0; i < claimDOs.length; i++) {
                if (claimDOs[i].isUserEditable()) {
                    claims.add(claimDOs[i]);
                }
            }
        } catch (Exception e) {
            return ERROR;
        }

        this.loadMessages();

        return SUCCESS;
    }

    public List<ClaimDO> getClaims() {
        return claims;
    }

    public void setClaims(List<ClaimDO> claims) {
        this.claims = claims;
    }

}