/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.StrutsStatics;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.UserStore;
import org.wso2.solutions.identity.admin.ClaimsAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;
import org.wso2.solutions.identity.user.ui.UIConstants;
import org.wso2.solutions.identity.users.IdentityDefaultRealm;
import org.wso2.solutions.identity.users.IdentityUserStoreAdmin;
import org.wso2.solutions.identity.users.IdentityUserStoreReader;
import org.wso2.usermanager.UserManagerException;

import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.validator.annotations.RequiredStringValidator;

public class AddUserProfileSubmitAction extends ManagedAction {

    /**
     * 
     */
    private static final long serialVersionUID = 6178599086703749725L;

    private String profileName;

    /**
     * This will get executed while adding a new user profile to an existing
     * user.
     */
    public String execute() throws Exception {

        String userName = null;
        HttpServletRequest request = null;
        ClaimsAdmin claimsAdmin = null;
        ClaimDO[] claimDOs = null;

        request = (HttpServletRequest) ActionContext.getContext().get(
                StrutsStatics.HTTP_REQUEST);

        userName = (String) request.getSession().getAttribute(UIConstants.USER);

        if (userName == null) {
            this.addErrorMessage(getText("invalid_user_login"));
            return ERROR;
        }

        if (profileName == null || profileName.trim().length() == 0) {
            this.addErrorMessage(getText("profile_name_required"));
            return ERROR;
        }

        claimsAdmin = new ClaimsAdmin();
        claimDOs = claimsAdmin.getAllMappedEnabledClaims();

        HashMap<String, String> props = new HashMap<String, String>();
        for (int i = 0; i < claimDOs.length; i++) {
            String uri = claimDOs[i].getUri();
            if (!uri.equals(IdentityConstants.CLAIM_PPID)) {

                if (claimDOs[i].isRequired()
                        && (request.getParameter(uri) == null || request
                                .getParameter(uri).trim().length() == 0)) {
                    this.addErrorMessage(getText("required_feilds_missing"));
                    return ERROR;
                }

                if (request.getParameter(uri) != null
                        && request.getParameter(uri).trim().length() > 0)
                    props.put(uri, request.getParameter(uri));
            }
        }

        try {

            IdentityDefaultRealm realm = null;
            IdentityUserStoreReader reader = null;
            IdentityUserStoreAdmin admin = null;

            realm = (IdentityDefaultRealm) UserStore.getInstance().getRealm();
            admin = realm.getIdentityUserStoreAdmin();
            reader = realm.getIdentityUserStoreReader();

            if (reader.isExistingUserProfile(userName, profileName)) {
                this.addErrorMessage(getText("user_profile_exists"));
                loadMessages();
                return ERROR;
            } else {
                admin.setUserProperties(userName, props, profileName);
            }

        } catch (UserManagerException e) {
            String msg = getText("error_unexpected", new String[] { e
                    .getMessage() });
            this.addErrorMessage(msg);
            return ERROR;
        }

        return SUCCESS;
    }

    @RequiredStringValidator(message = "", key = "profileName")
    public String getProfileName() {
        return profileName;
    }

    public void setProfileName(String profileName) {
        this.profileName = profileName;
    }

}