/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.StrutsStatics;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.admin.RegisteredInfoCardInfoAdmin;
import org.wso2.solutions.identity.admin.ReportAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ActionDO;
import org.wso2.solutions.identity.persistence.dataobject.RegisteredInfoCardInfoDO;
import org.wso2.solutions.identity.relyingparty.TokenVerifierConstants;
import org.wso2.solutions.identity.user.ui.UIConstants;
import org.wso2.solutions.identity.util.IdentityUtil;

import com.opensymphony.xwork2.ActionContext;

/**
 * Process an info card login
 */
public class InfoCardSubmitAction extends ManagedAction {

    private static final long serialVersionUID = 264026108621800214L;

    public String execute() throws Exception {
        ActionContext context = ActionContext.getContext();
        HttpServletRequest request = (HttpServletRequest) context
                .get(StrutsStatics.HTTP_REQUEST);

        String state = (String) request.getAttribute(TokenVerifierConstants.SERVLET_ATTR_STATE);
        if(state == null || (state!= null && state.equals(TokenVerifierConstants.STATE_FAILURE))) {
            return ERROR;
        }
        
        String ppid = (String) request
                .getAttribute(IdentityConstants.CLAIM_PPID);
        String issuerInfo = (String) request
                .getAttribute(TokenVerifierConstants.ISSUER_INFO);

        RegisteredInfoCardInfoAdmin admin = new RegisteredInfoCardInfoAdmin();
        RegisteredInfoCardInfoDO info = admin.getInfo(ppid);

        if (info != null && info.getIssuerInfo().equals(issuerInfo)) {
            context.getSession().put(UIConstants.USER, info.getUserId());
            context.getSession().put(UIConstants.PPID, ppid);
            ReportAdmin.record(info.getUserId(),
                    ActionDO.ACTION_USER_LOG_IN_CARD, "PPID=" + ppid);
            this.addInfoMessage(getText("login_successful",
                    new String[] { IdentityUtil
                            .getPPIDDisplayValue(ppid) }));
        } else {
            this.addErrorMessage(getText("invalid_card_login"));
            return ERROR;
        }
        return SUCCESS;
    }
}
