/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import org.apache.struts2.StrutsStatics;
import com.opensymphony.xwork2.ActionContext;
import com.opensymphony.xwork2.ActionSupport;

import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.admin.ReportAdmin;
import org.wso2.solutions.identity.persistence.dataobject.ActionDO;
import org.wso2.solutions.identity.relyingparty.RelyingPartyException;
import org.wso2.solutions.identity.user.ui.UIConstants;
import org.wso2.solutions.identity.user.ui.util.UserUtil;

public class OpenIDCallbackAction extends ActionSupport {

    private static final long serialVersionUID = 8513265604523184916L;

    /**
     * This will get executed after the user successfully authenticated to the
     * OpenID Provider
     */
    public String execute() throws Exception {

        ActionContext context = null;
        HttpServletRequest request = null;
        Map map = null;
        String openID = null;
        String userID = null;

        try {
            context = ActionContext.getContext();
            request = (HttpServletRequest) context
                    .get(StrutsStatics.HTTP_REQUEST);
            map = ActionContext.getContext().getSession();

            openID = (String) request
                    .getAttribute(IdentityConstants.OpenId.OPENID_IDENTIFIER);

            if (openID == null)
                return ERROR;

            // If this OpenID is not from WSO2 OP, we do not let him go in.
            // If this is from WSO2 OP, then user should be in our DB.
            userID = UserUtil.getUserName(openID);
            
            // Set the logged in user's id.
            map.put(UIConstants.USER, userID);
            
            String message = getText("successful_for", new String[] {userID});
            ReportAdmin.record(userID, ActionDO.ACTION_USER_LOG_IN_OPENID, message);

        } catch (RelyingPartyException e) {
            return ERROR;
        }
        return SUCCESS;
    }

}