/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.cards.CardIssuer;
import org.wso2.solutions.identity.relyingparty.RelyingPartyException;

public class OpenIDDownloadInfoCardAction extends DownloadInfoCardAction {

    private static final long serialVersionUID = 5406684555050588740L;

    /**
     * This will get executed during OpenID Information card download
     */
    public String execute() throws Exception {
        try {
            return super.execute();
        } catch (Exception e) {
            return ERROR;
        }
    }

    /**
     * Overrides the base class method with the CardIssuer related to OpenID
     * Information card
     */
    protected CardIssuer getCardIssuer() throws RelyingPartyException {

        CardIssuer issuer = null;
        try {
            issuer = new CardIssuer();
        } catch (IdentityProviderException e) {
            throw new RelyingPartyException(
                    IdentityConstants.ErrorCodes.CARD_ISSUER_INIT_FAILURE);
        }

        // We issue OpenIDInfoCards now - so our CardIssuer should support it.
        issuer.setIsOpenIdInfoCard(true);

        return issuer;
    }
}