/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.action;

import java.util.Map;

import org.wso2.solutions.identity.persistence.IPPersistenceManager;
import org.wso2.solutions.identity.persistence.dataobject.OpenIDUserRPDO;
import org.wso2.solutions.identity.user.ui.UIConstants;

import com.opensymphony.xwork2.ActionContext;

public class UpdateOpenIDUserRPAction extends ManagedAction {

    /**
     * 
     */
    private static final long serialVersionUID = 2085782218965074522L;

    private String rpUrl;

    private String operation;

    /**
     * This will get executed while updating user visited relying Parties
     */
    public String execute() throws Exception {

        if (rpUrl != null && operation != null && rpUrl.length() > 0
                && operation.length() > 0) {

            Map session = ActionContext.getContext().getSession();
            String user = (String) session.get(UIConstants.USER);

            if (user == null) {
                this.addErrorMessage(getText("invalid_user_login"));
                return ERROR;
            }

            IPPersistenceManager db = IPPersistenceManager
                    .getPersistanceManager();
            OpenIDUserRPDO[] rpdos = null;
            OpenIDUserRPDO rpdo = null;

            rpdos = db.getOpenIDUserRP(user, rpUrl);

            if (rpdos != null && rpdos.length > 0) {
                rpdo = rpdos[0];

                if (operation.equals(UIConstants.CLAIM_UPDATE_DISABLE)) {
                    rpdo.setIsTrustedAlways(false);
                } else if (operation.equals(UIConstants.CLAIM_UPDATE_ENABLE)) {
                    rpdo.setIsTrustedAlways(true);
                }

                db.update(rpdo);
            }
        }
        return SUCCESS;
    }

    public String getRpUrl() {
        return rpUrl;
    }

    public void setRpUrl(String rpUrl) {
        this.rpUrl = rpUrl;
    }

    public String getOperation() {
        return operation;
    }

    public void setOperation(String operation) {
        this.operation = operation;
    }

}