/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.user.ui.filter;

import org.wso2.solutions.identity.openid.OpenIDProvider;
import org.wso2.solutions.identity.user.ui.UIConstants;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

public class JSPFilter implements Filter {

    private FilterConfig filterConfig;

    public void destroy() {
    }

    public void doFilter(ServletRequest req, ServletResponse resp,
            FilterChain chain) throws IOException, ServletException {

        HttpServletRequest request = (HttpServletRequest) req;
        String path = request.getRequestURI();

        if (path.indexOf(UIConstants.OPENID_SERVER_PAGE) >= 0) {

            OpenIDProvider provider = new OpenIDProvider();

            provider.setAuthPage("OpenIDAuthentication.action");

            try {

                String response = null;
                response = provider.processRequest((HttpServletRequest) req,
                        (HttpServletResponse) resp);

                try {
                    new URL(response);
                    ((HttpServletResponse) resp).sendRedirect(response);
                    return;
                } catch (MalformedURLException ex) {
                    chain.doFilter(req, resp);
                }

            } catch (Exception e) {
                // TODO : need to work on exception handling                
            }
        } else if (path.indexOf(UIConstants.OPENID_USER_PAGE) >= 0) {
            chain.doFilter(req, resp);
        } else if (path.indexOf(UIConstants.JSP_ERROR_PAGE) < 0) {
            this.filterConfig.getServletContext().getRequestDispatcher(
                    UIConstants.JSP_ERROR_PAGE).forward(req, resp);
            return;
        } else {
            // Only allow access to error JSP
            chain.doFilter(req, resp);
        }

    }

    public void init(FilterConfig config) throws ServletException {
        this.filterConfig = config;
    }

}
